"""
/**********************************************************************
FILE     : $RCSfile: Zamino.py,v $
PURPOSE  : a (very basic) zope-wrapper for taminotalk Tamino XML DB interface
NOTES    : for comment
AUTHOR   : Stephen Schaper
COPYRIGHT: (c) 2001 by struktur AG
DATE     : 22OCT2001
LICENSE  : see LICENSE.TXT
REVISION : $Revision: 1.1 $
VERSION  : $Id: Zamino.py,v 1.1 2001/10/29 12:16:20 mache Exp $ (Author: $Author: mache $)

struktur AG            Phone: +49 711 8966560
Junghansstr. 5         Fax:   +49 711 89665610
70469 Stuttgart        email: info@struktur.de
GERMANY 

http://www.struktur.de
http://www.strukturag.com

**********************************************************************/

zamino is a minimal Zope product that implements an example interface between the
Tamino XML database (Software AG) and Zope. zamino supports XSLT of XML documents
stored in the Tamino DB. The XSLT translation results are cached and stored in a
zope object. 

Prequerisites for zamino:

- Zope 2.4.x
- taminotalk Client API for the Tamino XML database
  The API implements basic functions for access and query of Tamino, XSLT formatting
  of XML output and transaction handling.
- Transparent folders
- 4Suite 4DOM and 4XSLT
- Tamino XML database

Disclaimer

  THIS SOFTWARE IS PROVIDED BY STRUKTUR AG ``AS IS'' AND ANY
  EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STRUKTUR AG OR ITS
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  SUCH DAMAGE.

"""

__version__='$Revision: 1.1 $'[11:-2]


#standard import
from DateTime import DateTime
import time,re,string
from urllib import quote
from Globals import DTMLFile
#base classes
from OFS import SimpleItem,ZDOM,DTMLMethod
from Acquisition import Implicit
import Persistence
#classes for io
from StringIO import StringIO
from taminotalk import TaminoTalk

#
# XML, Tamino and Conversion :)
# a (very basic) zope-wrapper for taminotalk
#
class Zamino(Persistence.Persistent, SimpleItem.Item, Implicit):
    "XML, Tamino and Conversion (XTC)"

    meta_type='Zamino'


#
#zope management-options
#
    manage_options = (
        {'label': 'Properties', 'action': 'manage_editForm',},
        {'label': 'View', 'action': 'index_html',},
        {'label': 'View XML', 'action': 'manage_viewxmlForm',},
    )


#
# zope event handling
#
    def manage_afterAdd(self,item,container):
        """ manage_afterAdd """
        #After init and adding the object to the folder render the xml.
        #why? because the xsl is only visible/reachable after adding the object to the folder (yes, it's using Acquisition :)
        if self._initdone == 0:
            self._initdone = 1
            self._handlexmlfile_afterinit()

#
# taminotalk methods
#

#TODO: separate generic functions from object methods and attr.

    def getrendered(self):
        """ return self._rendered """
        return self._rendered

    def getxml(self, xquery=None):
        """ get the xml-object out of tamino """
        print 'TConnect: ', self._server, self._database, self.collection #debug
        t = TaminoTalk()
        t.connectTo(self._server, self._database, self.collection)
        t.setVerbose(1)  #debug
        if xquery is None:
            ret = t.xql(self.eigenquery).read()
        else:
            ret = t.xql(xquery).read()
        return ret

    def putxmlfile(self, xmlstring=None, xmlfile=''):
        """ put a xml-file into tamino-database """
        t = TaminoTalk()
        print 'TConnect: ', self._server, self._database, self.collection #debug
        t.connectTo(self._server, self._database, self.collection)
        t.setVerbose(1)  #debug
        if (type(xmlfile) is type('')) and (xmlstring is None):
            ttresponse = t.process(self.xmlfile , self.doctype)
        else:
            if xmlstring is not None:
                ttresponse = t.process(xmlstring , self._getdoctype(xmlstring))
            else:
                xmlfile=xmlfile.read()
                ttresponse = t.process(xmlstring , self._getdoctype(xmlfile))
        return ttresponse
               

    def renderxml(self, xquery=None):
        """ get xml from tamino and render it """
        print 'TConnect: ', self._server, self._database, self.collection #debug
        t = TaminoTalk()
        t.connectTo(self._server, self._database, self.collection)
        t.setVerbose(1) #debug
        if xquery is None:
            temp = t.xql(self.eigenquery).read()
        else:
            temp = t.xql(xquery).read()
        #print temp #debug
        self._rendered = t.applyXSLT( temp , StringIO(getattr(self, self.xslstyle)( self )))
        return self._rendered


#
# init and edit
#

    def __init__(self, id, title, xmlfile, xslstyle, dtmlstyle, taminohandler):
        """init item"""
        self.id = id
        self.title=title
        if type(xmlfile) is not type(''):
            xmlfile=xmlfile.read()
            self.xmlfile = xmlfile
        else:
            self.xmlfile = emptyXML()
        self.xmlfile = xmlfile
        self.xslstyle = xslstyle
        self.dtmlstyle = dtmlstyle
        self.taminohandler = taminohandler
        self._initdone = 0

    def manage_edit(self, title, xmlfile, xslstyle, dtmlstyle, taminohandler, REQUEST=None):
        """Edit Item"""
        self.title=title
        if type(xmlfile) is not type(''):
            xmlfile=xmlfile.read()
            self.xmlfile = xmlfile
        else:
            self.xmlfile = emptyXML()
        self.xslstyle = xslstyle
        self.dtmlstyle = dtmlstyle
        self.taminohandler = taminohandler
        self._handlexmlfile_afteredit()
        if REQUEST is not None:
            try: u=self.DestinationURL()
            except: u=REQUEST['URL1']
#            u="%s/%s" % (u,quote(self.id))
            REQUEST.RESPONSE.redirect(u+'/manage_editForm')

#
# private,protected and helper methods
#

    def emptyXML(self):
        """ emptyXML """
        #is used to represent an empty xml-upload
        return """<?xml version="1.0"?>
<null/>"""

    def nonValidXML(self):
        """ nonValidXML """
        #is used to represent an non valid xml-upload
        return """<?xml version="1.0"?>
<nonValid/>"""

    def _generate_eigenquery(self):
        """ generate eigenquery """
        #query for retrieving the xml-object that this zope-object is representing
        if (self.inoid is not None) and (self.doctype is not None):
            self.eigenquery = self.doctype + '[@ino:id="' + str(self.inoid) +'"]'
        else:
            self.eigenquery = None

    def _getdoctype(self, xmlstring=None):
        """ get the doctype of xml """
        #TODO: check this implementation
        try:
            if xmlstring is None:
                return re.search('<(\w+).*>', self.xmlfile).group(1)
            else:
                return re.search('<(\w+).*>', xmlstring).group(1)
        except:
           return None
        

    def _getcollection(self, xmlstring=None):
        """ get the collection for given xml """
        #TODO: get the right collection by context and matching the given doctype (till now: publish)
        return 'publish'

    def _getService(self):
        """get Tamino Service"""
        #TODO: create an object that represent this database related-stuff
        #self._server='10.1.1.11'
        #self._database='sp'

        self._server,self._database = getattr(self,self.taminohandler).giveService()


    def _handlexmlfile_afterinit(self):
        """ get all information ,put and render the xml """
        self._getService()
        self.doctype = self._getdoctype()
        self.collection = self._getcollection()
        #TODO: this could be far better and is propably not right working (compare string with method(which gives string)?)
        if self.xmlfile != self.emptyXML():
            tempresp = self.putxmlfile()
            if (tempresp[0] == 0):
                self.inoid = tempresp[2][0]
            else:
                self.inoid = None
                self.xmlfile = self.nonValidXML()
        self._generate_eigenquery()
        self._renderxmlfileafteraction()


    def _handlexmlfile_afteredit(self):
        """ get all information ,put and render the xml """
        self._getService()
        self.doctype = self._getdoctype()
        self.collection = self._getcollection()
        #TODO: what shall happen with the old xml?
        #TODO: this could be far better and is propably not right working (compare string with method(which gives string)?)
        if (self.xmlfile != self.emptyXML()) and (self.xmlfile != self.nonValidXML()) :
            tempresp = self.putxmlfile()
            if (tempresp[0] == 0):
                self.inoid = tempresp[2][0]
            self._generate_eigenquery()
            self._renderxmlfileafteraction()

    def _renderxmlfileafteraction(self):
        """ render the self.xmlfile  """ 
        t = TaminoTalk()
        t.setVerbose(1)  #debug
        self._rendered = t.applyXSLT( self.xmlfile , StringIO(getattr(self,self.xslstyle)( self )))
        #emtpy the container
        self.xmlfile = self.emptyXML()

#
# web interface 
#
 
    manage_editForm = DTMLFile('dtml/manage_editForm', globals())

    manage_viewxmlForm = DTMLFile('dtml/manage_viewxmlForm', globals())
 
    def index_html(self, client=None, REQUEST={}, RESPONSE=None, **kw):
        """index_html"""
        try:
            return apply(getattr(self,self.dtmlstyle),( self, REQUEST , RESPONSE),kw)
        except:
            try:
                return apply(getattr(self,'defauldtmlstyle'),( self, REQUEST , RESPONSE),kw)
            except:
                return self._rendered

#
# constructors
#

def manage_addZaminoAction(self ,id='Zamino-Document' , title='', xmlfile='', xslstyle='',dtmlstyle='defauldtmlstyle' , taminohandler='stdTamino' ,REQUEST=None):
    "Add a Zamino object to a folder."
    id=str(id)
    title=str(title)
    ob=Zamino(id, title, xmlfile, xslstyle,dtmlstyle,taminohandler)
    ob.title=title
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_editForm')

manage_addZaminoForm = DTMLFile('dtml/manage_addZaminoForm', globals())
   
