"""
/**********************************************************************
FILE     : $RCSfile: testcmd.py,v $
PURPOSE  : Command line interface for the Tamino XML database
NOTES    : Release version
AUTHOR   : Niels Mache
COPYRIGHT: (c) 2001, 2002 by struktur AG
DATE     : 16SEP2001
L.CHANGE : 04FEB2002
LICENSE  : see LICENSE.TXT
REVISION : $Revision: 1.4 $
VERSION  : $Id: testcmd.py,v 1.4 2002/02/04 16:54:27 mache Exp $ (Author: $Author: mache $)

struktur AG            Phone: +49 711 8966560
Junghansstr. 5         Fax:   +49 711 89665610
70469 Stuttgart        email: info@struktur.de
GERMANY 

http://www.struktur.de
http://www.strukturag.com

**********************************************************************/

Command line interface for the Tamino XML database

HISTORY:

16SEP2001:
  First release.

04FEB2002:
  Updated process and define commands (taminotalk process and define commands return an unparsed stream now).  



Disclaimer

  THIS SOFTWARE IS PROVIDED BY STRUKTUR AG ``AS IS'' AND ANY
  EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
  PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STRUKTUR AG OR ITS
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  SUCH DAMAGE.
"""

# import DOM
import xml.dom
import xml.dom.ext
from xml.dom.ext.reader import PyExpat
from xml.dom.minidom import parse, parseString

from taminotalk import TaminoTalk


class CmdT:
    def __init__(self):
        self.database = None
        self.server = None
        self.collection = None
        self.doctype = None
        self.cursor = None
        self.count = None
        self.output = None


    def usage(self):
        print """
Tamino command line interface.
(c) 2001, 2002 by struktur AG
www.struktur.de, www.strukturag.com
All rights reserved.

Usage: tamino -[hpfqosdctge]  <query or files or dirs>
              --[help|output|query|server|database|doctype|process|collection|get=<ids>|define|delete] <query or files or dirs>

-s <server>     --server=<server>           server name or IP address
-d <database>   --database=<database>       database name
-c <collection> --collection=<collection>   collection name
-t <doctype>    --doctype=<doctype>         doctype (optional)

-p --process <files or directory>           process files (load data into Tamino) a directory can be
                                            given to process all files in the directory
-q --query=<XPath query>                    XPath query
-g --get=<ino:ids>                          get documents by ID, multiple ID's can be given
-e --delete=<XPath query>                   delete documents selected by XPath query
-f --define                                 define collections or doctypes
-u --undefine=<collection or doctype>       delete collection or doctype. Use with care!
-o <filename>   --output=<filename>         output to file
-h --help                                   print this help

"""

    def processFiles(self, filesOrDirs, cmd):
        import os.path

        for fd in filesOrDirs:
            if os.path.isdir(fd):
                dir = fd
                filelist = os.listdir(fd)
            else:
                dir = ''
                filelist = [ fd ]

            for f in filelist:
                file = os.path.normpath(os.path.join(dir, f))
                if os.path.isfile(file):
                    try:
                        print 'Process file: ', file
                        fp = open(file, 'r')
                        ret = cmd(fp)
                        fp.close()
                        if ret != 0:
                            return -1  # return error code
                    except IOError, msg:
                        print >> sys.stderr, "Error opening file %s: %s" % (file, msg)
                        sys.exit(1)
        return 0

    def __dummy(self, f):
        print f
        return 0

    def __process(self, fp):
        response = self.connect.process(fp, self.doctype)

        # parse response
        if response is not None:
            theDOM = self.connect.getDOM(response)
            response.close()
            returnvalue, messageline, docIDs, resultDOMs, cursor = self.connect.parseTaminoResults(theDOM)
            # print results
            print >> self.output, '-----------------------------------------------------'
            print >> self.output, 'Return value: ', returnvalue
            print >> self.output, 'Message line: ', messageline
            print >> self.output, 'Document ID : ', docIDs

            #print >> self.output, xml.dom.ext.PrettyPrint(theDOM)
        else:
            print  >> self.output, "ERROR: Process command failed"

        return returnvalue


    def doProcess(self, fileORdir):
        print 'doProcess: ', fileORdir

        self.connect = TaminoTalk()
        self.connect.connectTo(self.server, self.database, self.collection)
        return self.processFiles(fileORdir, self.__process)
    #   self.processFiles(fileORdir, self.__dummy)


    def doQuery(self, query):
        print 'doQuery: ', query

        self.connect = TaminoTalk()
        self.connect.connectTo(self.server, self.database, self.collection)
        theDOM = self.connect.getDOM(self.connect.xql(query, self.cursor, self.count))
        returnvalue, messageline, docIDs, resultDOMs, cursor = self.connect.parseTaminoResults(theDOM)
        print >> self.output, '-----------------------------------------------------'
        print >> self.output, 'Return value: ', returnvalue
        print >> self.output, 'Message line: ', messageline
        print >> self.output, 'Document ID : ', docIDs
        print >> self.output, '\n\n'
        xml.dom.ext.PrettyPrint(theDOM)

    def doUndefine(self, undefine):
        print 'doUndefine: ', undefine

        self.connect = TaminoTalk()
        self.connect.connectTo(self.server, self.database, self.collection)
        returnvalue = self.connect.undefine(undefine, 120664)
        print >> self.output, '-----------------------------------------------------'
        print >> self.output, 'Return value: ', returnvalue

    def doDelete(self, delete):
        print 'doDelete: ', delete

        self.connect = TaminoTalk()
        self.connect.connectTo(self.server, self.database, self.collection)
        self.connect.setVerbose(1)
        returnvalue = self.connect.delete(delete)
        print >> self.output, '-----------------------------------------------------'
        print >> self.output, 'Return value: ', returnvalue

    def doGet(self, get):
#       print 'doGet: ', get

        self.connect = TaminoTalk()
        self.connect.connectTo(self.server, self.database, self.collection)

        for id in get:
#       print 'ID: ', id
            fp = self.connect.getContentByID(int(id), self.doctype)
            if self.output is not None:
                self.output.write(fp.read())
            else:
                print fp.read()

            fp.close()

# Define a collection or schema.
    def __define(self, fp):
        returnvalue, messageline, docIDs, theDOM = self.connect.define(fp)
        response = self.connect.define(fp)

        # parse response
        if response is not None:
            theDOM = self.connect.getDOM(response)
            response.close()
            returnvalue, messageline, docIDs, resultDOMs, cursor = self.connect.parseTaminoResults(theDOM)
            # print results
            print >> self.output, '-----------------------------------------------------'
            print >> self.output, 'Return value: ', returnvalue
            print >> self.output, 'Message line: ', messageline
            print >> self.output, 'Document ID : ', docIDs
        else:
            print  >> self.output, "ERROR: Define command failed"

        return returnvalue

    def doDefine(self, define):
        print 'doDefine: ', define

        self.connect = TaminoTalk()
        self.connect.connectTo(self.server, self.database, self.collection)
        return self.processFiles(define, self.__define)


    def main(self):
        import getopt, sys
        
        try:
            opts, args = getopt.getopt(sys.argv[1:],
                                       "hpfq:o:s:d:c:t:ge:u:x",
                                       ["help", "output=", "query=",
                                        "server=", "database=", "collection=",
                                        "doctype=", "process", "get", "delete=",
                                        "undefine=", "filebox", "define"]
                                       )
        except getopt.GetoptError:
            # print help information and exit:
            self.usage()
            sys.exit(2)

        query, process, define, delete, undefine, get = (None, None, None, None, None, None)
        for cmd, arg in opts:
            if cmd in ('-h', '--help'):
                self.usage()
                sys.exit()
            if cmd in ('-o', '--output'):
                # redirect output to file
                print 'Output to file'
                try:
                    self.output = open(arg, 'w')
                except IOError, msg:
                    print >> sys.stderr, "Error redirecting output %s: %s" % (arg, msg)
                    sys.exit(1)
                continue
            if cmd in ('-x', '--filebox'):
                import FileDialogs
                # with fileselectorbox
                args = ExecuteOpenDialog(title = "Please select file(s) to open...",
                                         flags = win32con.OFN_ALLOWMULTISELECT | win32con.OFN_FILEMUSTEXIST,
                                         filter = "XML Files|*.xml|All Files|*.*||")
                print 'args: ', args
                continue
            if cmd in ('-s', '--server'):
                self.server = arg  # get the server URL
                continue
            if cmd in ('-d', '--database'):
                self.database = arg  # get the database
                continue
            if cmd in ('-c', '--collection'):
                self.collection = arg  # get the collection
                continue
            if cmd in ('-t', '--doctype'):
                self.doctype = arg  # get the doctype (optional)
                continue
            if cmd in ('-g', '--get'):
                get = args  # get a document by ID
                continue
            if cmd in ('-q', '--query'):
                # do a XPath query
                if process is not None or define is not None:
                    self.usage()  # query and process simultanously
                    sys.exit(2)
                query=arg
                continue
            if cmd in ('-e', '--delete'):
                # delete documents by query
                if process is not None or define is not None:
                    self.usage()  # query and process simultanously
                    sys.exit(2)
                delete=arg
                continue
            if cmd in ('-u', '--undefine'):
                # undefine collection or doctype
                if process is not None or define is not None:
                    self.usage()  # query and process simultanously
                    sys.exit(2)
                undefine=arg
                continue
            if cmd in ('-p', '--process'):
                # store data in Tamino. Data is given by a file or directory. 
                if query is not None or define is not None:
                    self.usage()  # query and process simultanously 
                    sys.exit(2)
                process = args
                continue
            if cmd in ('-f', '--define'):
                # define a collection or doctype. Schema is given by a file. 
                if query is not None or process is not None:
                    self.usage()  # query and process simultanously 
                    sys.exit(2)
                define = args
                continue
            if cmd in ('-g', '--get'):
                # get documents by ID. 
                get = args
                continue

#       print 'server, database, collection, doctype', self.server, self.database, self.collection, self.doctype

        if self.server is None or self.database is None or self.collection is None:
            print >> sys.stderr, 'Error: server, database and collection must be given'
            self.usage()
            sys.exit(2)
            
        if query is not None:
            self.doQuery(query)
        if delete is not None:
            self.doDelete(delete)
        if process is not None:
            self.doProcess(process)
        if define is not None:
            self.doDefine(define)
        if undefine is not None:
            self.doUndefine(undefine)
        if get is not None:
            self.doGet(get)

        if self.output is not None:
            self.output.close()


# -------------------------------------------------------------------------------
# main class
#

if __name__ == '__main__':
    t = CmdT()
    t.main()
